@@ -38,14 +38,16 @@ enum {
   PROP_URI
 };
 
-static void load_openoffice (SushiPdfLoader *self);
+static void load_libreoffice (SushiPdfLoader *self);
 
 struct _SushiPdfLoaderPrivate {
   EvDocument *document;
   gchar *uri;
   gchar *pdf_path;
 
-  GPid unoconv_pid;
+  gboolean checked_libreoffice_flatpak;
+  gboolean have_libreoffice_flatpak;
+  GPid libreoffice_pid;
 };
 
 static void
@@ -81,42 +83,42 @@ load_pdf (SushiPdfLoader *self,
 }
 
 static void
-openoffice_missing_unoconv_ready_cb (GObject *source,
-                                     GAsyncResult *res,
-                                     gpointer user_data)
+libreoffice_missing_ready_cb (GObject *source,
+                              GAsyncResult *res,
+                              gpointer user_data)
 {
   SushiPdfLoader *self = user_data;
   GError *error = NULL;
 
   g_dbus_connection_call_finish (G_DBUS_CONNECTION (source), res, &error);
   if (error != NULL) {
-    /* can't install unoconv with packagekit - nothing else we can do */
+    /* can't install libreoffice with packagekit - nothing else we can do */
     /* FIXME: error reporting! */
-    g_warning ("unoconv not found, and PackageKit failed to install it with error %s",
+    g_warning ("libreoffice not found, and PackageKit failed to install it with error %s",
                error->message);
     return;
   }
 
-  /* now that we have unoconv installed, try again loading the document */
-  load_openoffice (self);
+  /* now that we have libreoffice installed, try again loading the document */
+  load_libreoffice (self);
 }
 
 static void
-openoffice_missing_unoconv (SushiPdfLoader *self)
+libreoffice_missing (SushiPdfLoader *self)
 {
   GApplication *app = g_application_get_default ();
   GtkWidget *widget = GTK_WIDGET (gtk_application_get_active_window (GTK_APPLICATION (app)));
   GDBusConnection *connection = g_application_get_dbus_connection (app);
   guint xid = 0;
   GdkWindow *gdk_window;
-  const gchar *unoconv_path[2];
+  const gchar *libreoffice_path[2];
 
   gdk_window = gtk_widget_get_window (widget);
   if (gdk_window != NULL)
     xid = GDK_WINDOW_XID (gdk_window);
 
-  unoconv_path[0] = "/usr/bin/unoconv";
-  unoconv_path[1] = NULL;
+  libreoffice_path[0] = "/usr/bin/libreoffice";
+  libreoffice_path[1] = NULL;
 
   g_dbus_connection_call (connection,
                           "org.freedesktop.PackageKit",
@@ -125,25 +127,25 @@ openoffice_missing_unoconv (SushiPdfLoader *self)
                           "InstallProvideFiles",
                           g_variant_new ("(u^ass)",
                                          xid,
-                                         unoconv_path,
+                                         libreoffice_path,
                                          "hide-confirm-deps"),
                           NULL, G_DBUS_CALL_FLAGS_NONE,
                           G_MAXINT, NULL,
-                          openoffice_missing_unoconv_ready_cb,
+                          libreoffice_missing_ready_cb,
                           self);
 }
 
 static void
-unoconv_child_watch_cb (GPid pid,
-                        gint status,
-                        gpointer user_data)
+libreoffice_child_watch_cb (GPid pid,
+                            gint status,
+                            gpointer user_data)
 {
   SushiPdfLoader *self = user_data;
   GFile *file;
   gchar *uri;
 
   g_spawn_close_pid (pid);
-  self->priv->unoconv_pid = -1;
+  self->priv->libreoffice_pid = -1;
 
   file = g_file_new_for_path (self->priv->pdf_path);
   uri = g_file_get_uri (file);
@@ -153,74 +155,156 @@ unoconv_child_watch_cb (GPid pid,
   g_free (uri);
 }
 
-static void
-load_openoffice (SushiPdfLoader *self)
+#define LIBREOFFICE_FLATPAK "org.libreoffice.LibreOffice"
+
+static gboolean
+check_libreoffice_flatpak (SushiPdfLoader *self,
+                           const gchar    *flatpak_path)
 {
-  gchar *doc_path, *pdf_path, *tmp_name, *tmp_path, *quoted_path;
-  GFile *file;
-  gboolean res;
-  gchar *cmd;
-
-  gint argc;
-  GPid pid;
-  gchar **argv = NULL;
+  const gchar *check_argv[] = { flatpak_path, "info", LIBREOFFICE_FLATPAK, NULL };
+  gboolean ret;
+  gint exit_status = -1;
   GError *error = NULL;
-  const gchar *unoconv_path;
 
-  unoconv_path = g_find_program_in_path ("unoconv");
-  if (unoconv_path == NULL) {
-      openoffice_missing_unoconv (self);
+  if (self->priv->checked_libreoffice_flatpak)
+    return self->priv->have_libreoffice_flatpak;
+
+  self->priv->checked_libreoffice_flatpak = TRUE;
+
+  ret = g_spawn_sync (NULL, (gchar **) check_argv, NULL,
+                      G_SPAWN_DEFAULT, NULL, NULL,
+                      NULL, NULL,
+                      &exit_status, &error);
+
+  if (ret) {
+    GError *child_error = NULL;
+    if (g_spawn_check_exit_status (exit_status, &child_error)) {
+        g_debug ("Found LibreOffice flatpak!");
+        self->priv->have_libreoffice_flatpak = TRUE;
+    } else {
+        g_debug ("LibreOffice flatpak not found, flatpak info returned %i (%s)",
+                 exit_status, child_error->message);
+        g_clear_error (&child_error);
+    }
+  } else {
+    g_warning ("Error while checking for LibreOffice flatpak: %s",
+               error->message);
+    g_clear_error (&error);
+  }
+
+  return self->priv->have_libreoffice_flatpak;
+}
+
+static void
+load_libreoffice (SushiPdfLoader *self)
+{
+  gchar *flatpak_path, *libreoffice_path = NULL;
+  gboolean use_flatpak = FALSE;
+  GFile *file;
+  gchar *doc_path, *doc_name, *tmp_name, *pdf_dir;
+  gchar *flatpak_doc = NULL, *flatpak_dir = NULL;
+  gboolean res;
+  GPid pid;
+  GError *error = NULL;
+  const gchar *argv = NULL;
+
+  flatpak_path = g_find_program_in_path ("flatpak");
+  if (flatpak_path != NULL)
+    use_flatpak = check_libreoffice_flatpak (self, flatpak_path);
+
+  if (!use_flatpak) {
+    libreoffice_path = g_find_program_in_path ("libreoffice");
+    if (libreoffice_path == NULL) {
+      libreoffice_missing (self);
+      g_free (flatpak_path);
       return;
+    }
   }
 
   file = g_file_new_for_uri (self->priv->uri);
   doc_path = g_file_get_path (file);
-  quoted_path = g_shell_quote (doc_path);
-
+  doc_name = g_file_get_basename (file);
   g_object_unref (file);
-  g_free (doc_path);
 
-  tmp_name = g_strdup_printf ("sushi-%d.pdf", getpid ());
-  tmp_path = g_build_filename (g_get_user_cache_dir (), "sushi", NULL);
-  self->priv->pdf_path = pdf_path =
-    g_build_filename (tmp_path, tmp_name, NULL);
-  g_mkdir_with_parents (tmp_path, 0700);
+  /* libreoffice --convert-to replaces the extension with .pdf */
+  tmp_name = g_strrstr (doc_name, ".");
+  if (tmp_name)
+    *tmp_name = '\0';
+  tmp_name = g_strdup_printf ("%s.pdf", tmp_name);
+  g_free (doc_name);
 
-  cmd = g_strdup_printf ("unoconv -f pdf -o %s %s", pdf_path, quoted_path);
+  pdf_dir = g_build_filename (g_get_user_cache_dir (), "sushi", NULL);
+  self->priv->pdf_path = g_build_filename (pdf_dir, tmp_name, NULL);
+  g_mkdir_with_parents (pdf_dir, 0700);
 
   g_free (tmp_name);
-  g_free (tmp_path);
-  g_free (quoted_path);
 
-  res = g_shell_parse_argv (cmd, &argc, &argv, &error);
-  g_free (cmd);
+  if (use_flatpak) {
+    flatpak_doc = g_strdup_printf ("--filesystem=%s:ro", doc_path);
+    flatpak_dir = g_strdup_printf ("--filesystem=%s", pdf_dir);
 
-  if (!res) {
-    g_warning ("Error while parsing the unoconv command line: %s",
-               error->message);
-    g_error_free (error);
+    const gchar *flatpak_argv[] = {
+      NULL, /* to be replaced with flatpak binary */
+      "run", "--command=/app/libreoffice/program/soffice",
+      "--nofilesystem=host",
+      NULL, /* to be replaced with filesystem permissions to read document */
+      NULL, /* to be replaced with filesystem permissions to write output */
+      LIBREOFFICE_FLATPAK,
+      "--convert-to", "pdf",
+      "--outdir", NULL, /* to be replaced with output dir */
+      NULL, /* to be replaced with input file */
+      NULL
+    };
 
-    return;
+    flatpak_argv[0] = flatpak_path;
+    flatpak_argv[4] = flatpak_doc;
+    flatpak_argv[5] = flatpak_dir;
+    flatpak_argv[10] = pdf_dir;
+    flatpak_argv[11] = doc_path;
+
+    argv = (const gchar *) &flatpak_argv[0];
+  } else {
+    const gchar *libreoffice_argv[] = {
+      NULL, /* to be replaced with binary */
+      "--convert-to", "pdf",
+      "--outdir", NULL, /* to be replaced with output dir */
+      NULL, /* to be replaced with input file */
+      NULL
+    };
+
+    libreoffice_argv[0] = libreoffice_path;
+    libreoffice_argv[4] = pdf_dir;
+    libreoffice_argv[5] = doc_path;
+
+    argv = (const gchar *) &libreoffice_argv[0];
   }
 
-  res = g_spawn_async (NULL, argv, NULL,
-                       G_SPAWN_DO_NOT_REAP_CHILD |
-                       G_SPAWN_SEARCH_PATH,
+  tmp_name = g_strjoinv (" ", (gchar **) argv);
+  g_debug ("Executing LibreOffice command: %s", tmp_name);
+  g_free (tmp_name);
+
+  res = g_spawn_async (NULL, (gchar **) argv, NULL,
+                       G_SPAWN_DO_NOT_REAP_CHILD,
                        NULL, NULL,
                        &pid, &error);
 
-  g_strfreev (argv);
+  g_free (pdf_dir);
+  g_free (doc_path);
+  g_free (libreoffice_path);
+  g_free (flatpak_path);
+  g_free (flatpak_doc);
+  g_free (flatpak_dir);
 
   if (!res) {
-    g_warning ("Error while spawning unoconv: %s",
+    g_warning ("Error while spawning libreoffice: %s",
                error->message);
     g_error_free (error);
 
     return;
   }
 
-  g_child_watch_add (pid, unoconv_child_watch_cb, self);
-  self->priv->unoconv_pid = pid;
+  g_child_watch_add (pid, libreoffice_child_watch_cb, self);
+  self->priv->libreoffice_pid = pid;
 }
 
 static gboolean
@@ -269,7 +353,7 @@ query_info_ready_cb (GObject *obj,
   if (content_type_is_native (content_type))
     load_pdf (self, self->priv->uri);
   else
-    load_openoffice (self);
+    load_libreoffice (self);
 
   g_object_unref (info);
 }
@@ -310,9 +394,9 @@ sushi_pdf_loader_cleanup_document (SushiPdfLoader *self)
     g_free (self->priv->pdf_path);
   }
 
-  if (self->priv->unoconv_pid != -1) {
-    kill (self->priv->unoconv_pid, SIGKILL);
-    self->priv->unoconv_pid = -1;
+  if (self->priv->libreoffice_pid != -1) {
+    kill (self->priv->libreoffice_pid, SIGKILL);
+    self->priv->libreoffice_pid = -1;
   }
 }
 
@@ -406,7 +490,7 @@ sushi_pdf_loader_init (SushiPdfLoader *self)
     G_TYPE_INSTANCE_GET_PRIVATE (self,
                                  SUSHI_TYPE_PDF_LOADER,
                                  SushiPdfLoaderPrivate);
-  self->priv->unoconv_pid = -1;
+  self->priv->libreoffice_pid = -1;
 }
 
 SushiPdfLoader *
